<?php

namespace Drupal\Tests\ldap_sso_auth\Unit;

use Drupal\ldap_sso_auth\LdapSsoAuthAuthentication;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Tests\UnitTestCase;
use Symfony\Component\HttpFoundation\Request;

/**
 * Tests the LdapSsoAuthAuthentication service.
 *
 * @group ldap_sso_auth
 * @coversDefaultClass \Drupal\ldap_sso_auth\LdapSsoAuthAuthentication
 */
class LdapSsoAuthAuthenticationTest extends UnitTestCase {

  /**
   * Tests LdapSsoAuthAuthentication::applies() method.
   *
   * @covers ::applies
   * @dataProvider providerApplies
   */
  public function testApplies($session, $pathInfo, $ssoVariable, $ssoValue, $expectedResult) {
    $container = new ContainerBuilder();
    $config = [
      'ldap_sso_auth.settings' => [
        'ssoVariable' => $ssoVariable,
        'ssoExcludedPaths' => [
          '/path/excluded',
          '<front>',
        ],
        'logoutRedirectPath' => '/user/login',
      ],
      'system.site' => [
        'frontpage' => '/frontpage',
      ],
    ];
    $configFactory = $this->getConfigFactoryStub($config);
    $container->set('config.factory', $configFactory);

    $entity_type_manager = $this->createMock('Drupal\Core\Entity\EntityTypeManagerInterface');
    $container->set('entity_type.manager', $entity_type_manager);

    $ldap_authentication_login_validator = $this->createMock('Drupal\ldap_authentication\Controller\LoginValidatorSso');
    $container->set('ldap_authentication.login_validator_sso', $ldap_authentication_login_validator);

    $ldap_log = $this->createMock('Drupal\ldap_servers\Logger\LdapDetailLog');
    $container->set('ldap.detail_log', $ldap_log);

    $request = $this->createMock(Request::class);
    $request->method('getSession')->willReturn($session);
    $request->method('getPathInfo')->willReturn($pathInfo);
    $request->server = $this->createMock('Symfony\Component\HttpFoundation\ServerBag');
    $request->server->method('get')->with($ssoVariable)->willReturn($ssoValue);

    $service = new LdapSsoAuthAuthentication($container);

    $this->assertEquals($expectedResult, $service->applies($request));
  }

  /**
   * Provides data to test LdapSsoAuthAuthentication::applies() method.
   */
  public function providerApplies() {
    return [
      [
        $this->createMock('Symfony\Component\HttpFoundation\Session\SessionInterface'),
        '/path',
        'SSO_VAR',
        'SSO_VALUE',
        TRUE,
      ],
      [
        $this->createMock('Symfony\Component\HttpFoundation\Session\SessionInterface'),
        '/user/login',
        'SSO_VAR',
        'SSO_VALUE',
        FALSE,
      ],
      [
        NULL,
        '/path',
        'SSO_VAR',
        'SSO_VALUE',
        TRUE,
      ],
      [
        $this->createMock('Symfony\Component\HttpFoundation\Session\SessionInterface'),
        '/path/excluded',
        'SSO_VAR',
        'SSO_VALUE',
        FALSE,
      ],
      [
        NULL,
        '/path/excluded',
        'SSO_VAR',
        'SSO_VALUE',
        FALSE,
      ],
      [
        $this->createMock('Symfony\Component\HttpFoundation\Session\SessionInterface'),
        '/frontpage',
        'SSO_VAR',
        'SSO_VALUE',
        FALSE,
      ],
      [
        NULL,
        '/frontpage',
        'SSO_VAR',
        'SSO_VALUE',
        FALSE,
      ],
    ];
  }

}
